@@ -1,4 +1,4 @@
-# Copyright 2012-2017, Andrey Kislyuk and argcomplete contributors.
+# Copyright 2012-2019, Andrey Kislyuk and argcomplete contributors.
 # Licensed under the Apache License. See https://github.com/kislyuk/argcomplete for more info.
 
 from __future__ import absolute_import, division, print_function, unicode_literals
@@ -8,7 +8,7 @@ from . import completers, my_shlex as shlex
 from .compat import USING_PYTHON2, str, sys_encoding, ensure_str, ensure_bytes
 from .completers import FilesCompleter, SuppressCompleter
 from .my_argparse import IntrospectiveArgumentParser, action_is_satisfied, action_is_open, action_is_greedy
-from .shellintegration import shellcode
+from .shell_integration import shellcode
 
 _DEBUG = "_ARC_DEBUG" in os.environ
 
@@ -213,6 +213,10 @@ class CompletionFinder(object):
         start = int(os.environ["_ARGCOMPLETE"]) - 1
         comp_words = comp_words[start:]
 
+        if cword_prefix and cword_prefix[0] in self._parser.prefix_chars and "=" in cword_prefix:
+            # Special case for when the current word is "--optional=PARTIAL_VALUE". Give the optional to the parser.
+            comp_words.append(cword_prefix.split("=", 1)[0])
+
         debug("\nLINE: {!r}".format(comp_line),
               "\nPOINT: {!r}".format(comp_point),
               "\nPREQUOTE: {!r}".format(cword_prequote),
@@ -248,7 +252,8 @@ class CompletionFinder(object):
 
         self.completing = False
 
-        # key: complete word, value: description.
+        if "--" in comp_words:
+            self.always_complete_options = False
 
         completions = self.collect_completions(active_parsers, parsed_args, cword_prefix, debug)
         completions = self.filter_completions(completions)
@@ -377,6 +382,7 @@ class CompletionFinder(object):
         debug("Active actions (L={l}): {a}".format(l=len(parser.active_actions), a=parser.active_actions))
 
         isoptional = cword_prefix and cword_prefix[0] in parser.prefix_chars
+        optional_prefix = ""
         greedy_actions = [x for x in parser.active_actions if action_is_greedy(x, isoptional)]
         if greedy_actions:
             assert len(greedy_actions) == 1, "expect at most 1 greedy action"
@@ -386,8 +392,14 @@ class CompletionFinder(object):
             self._display_completions = {}
             completions = []
         elif isoptional:
-            # Only run completers if current word does not start with - (is not an optional)
-            return completions
+            if "=" in cword_prefix:
+                # Special case for when the current word is "--optional=PARTIAL_VALUE".
+                # The completer runs on PARTIAL_VALUE. The prefix is added back to the completions
+                # (and chopped back off later in quote_completions() by the COMP_WORDBREAKS logic).
+                optional_prefix, _, cword_prefix = cword_prefix.partition("=")
+            else:
+                # Only run completers if current word does not start with - (is not an optional)
+                return completions
 
         complete_remaining_positionals = False
         # Use the single greedy action (if there is one) or all active actions.
@@ -438,6 +450,8 @@ class CompletionFinder(object):
                         if self.validator(next_completion, cword_prefix):
                             self._display_completions.update({next_completion: ""})
                             completions.append(next_completion)
+                if optional_prefix:
+                    completions = [optional_prefix + "=" + completion for completion in completions]
                 debug("Completions:", completions)
         return completions
 
@@ -537,8 +551,8 @@ class CompletionFinder(object):
         elif cword_prequote == '"':
             special_chars += '"`$!'
 
-        if os.environ.get("_ARGCOMPLETE_SHELL") == "tcsh":
-            # tcsh escapes special characters itself.
+        if os.environ.get("_ARGCOMPLETE_SHELL") in ("tcsh", "fish"):
+            # tcsh and fish escapes special characters itself.
             special_chars = ""
         elif cword_prequote == "'":
             # Nothing can be escaped in single quotes, so we need to close
